local LDB = LibStub("LibDataBroker-1.1", true)
local L = LibStub:GetLibrary("AceLocale-3.0" ):GetLocale("WTFrames")
local WTFrames = LibStub("AceAddon-3.0"):NewAddon("WTFrames", "AceConsole-3.0")
local SML

local oUF = _G["oUF"] 

if not oUF then
	oUF = oUFembed
	oUFembed = nil
end
		
local backdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		insets = {left = -1.5, right = -1.5, top = -1.5, bottom = -1.5},
}	
-- local statusBarTexture = [[Interface\Addons\WTFrames\media\statusbar]]
local borderTexture = 'Interface\\Addons\\WTFrames\\media\\border'
-- local borderTexture = 'Blizzard Tooltip'



local mTypeTexture = media and media.MediaType and media.MediaType.STATUSBAR or "statusbar"
local mTypeFont = media and media.MediaType and media.MediaType.FONT or "font"
local mTypeBorder = media and media.MediaType and media.MediaType.BORDER or "border"

local anchorRef = {}
local initLDB = {}
local InitConfig = {}
local maintankFrame = {}

local tempConfig = {}
local currentConfigVersion = 4
local textures = {}
local fonts = {}

local TankFrames = {}

function InitConfig()
	WTFramesConfig = {
		configVersion = currentConfigVersion,
		options = {
			ShowRaidIcons = true,
			OnlyShowBlizzardTanks = false,
			BarWidth = 120,
			BarHeight = 20,
			PowerBarHeightFactor = 0.25,
			GrowUpwards = false,
			GrowLeft = false,
			ShowPowerBars = true,
			ShowTargetTarget = true,
			IconSize = 16,
			YOffset = 2.9,
			TextureName = "WTFramesDefault",
			FontName = "WTFramesDefault",
			FontSize = 11,
		},
		frame = {
			point = "BOTTOMLEFT",
			x = 50,
			y = 500,
		}							
	}	
end

function WTFrames:Debug(name) 
	if(name == "mt") then
		return maintankFrame
	end
	if(name == "tf") then
		return TankFrames
	end
end

local function GetOption(name)
	local returnValue
	
	if (tempConfig[name] == nil) then		
		returnValue = WTFramesConfig.options[name]
	else
		returnValue = tempConfig[name]
	end
	
	return returnValue
end
	
local function loadOptions() 
	options = {	
		name = L["WTFrames"],
		type = "group",
		args = {
			confdesc = {
				order = 1,
				type = "description",
				name = string.format("Simple oRA3 Maintank Target Frames - Version: %s", GetAddOnMetadata("WTFrames", "@project-revision@") or "N/A"),
				width = "full",
				cmdHidden = true
			},
			generalOptions = {
				type = "group",
				name = "General Options",
				inline = true,
				order = 2,
				args = {	
					lockFrames = {
						order = 1, 
						type = "toggle", 
						width = "full",
						name = L["Lock frames"],
						desc = L["Lock frames"],
						get = function() return WTFrames:IsLocked() end,
						set = function(_,v) 
							if v then 
								WTFrames:Lock() 
							else 
								WTFrames:UnLock()
							end
						end,
					},		
					onlyShowBlizzardTanks = {
						order = 2, 
						type = "toggle", 
						width = "full",
						name = L["Only show Blizzard Maintanks"],
						desc = L["Doesn't show any personal tanks - only Blizzard tanks (oRA3 shield icon)"],
						get = function() return WTFramesConfig.options.OnlyShowBlizzardTanks end,
						set = function(_,v) WTFramesConfig.options.OnlyShowBlizzardTanks = v 
							WTFrames:UpdateMaintankList()
						end,
					},
				}
			},
			options = {
				type = "group",
				name = "Frame Options",
				inline = true,
				order = 3,
				args = {	
					growUpwards = {
						order = 1, 
						type = "toggle", 
						width = "full",
						name = "Grow upwards",
						desc = L["Grow upwards"],
						--disabled = true,
						get = function() return GetOption("GrowUpwards") end,
						set = function(_,v) tempConfig["GrowUpwards"] = v 
							--WTFrames:UpdateConfigOTF()
						end,
					},	
					growLeft = {
						order = 2, 
						type = "toggle", 
						width = "full",
						name = "Grow left",
						desc = L["Grow left"],
						--disabled = true,
						get = function() return GetOption("GrowLeft") end,
						set = function(_,v) tempConfig["GrowLeft"] = v 
							--WTFrames:UpdateConfigOTF()
						end,
					},						
					showPowerBars = {
						order = 3, 
						type = "toggle", 
						width = "full",
						name = L["Show power bars"],
						desc = L["Show power bars"],
						get = function() return GetOption("ShowPowerBars") end,
						set = function(_,v) tempConfig["ShowPowerBars"] = v 
							--WTFrames:UpdateConfig()
						end,
					},							
					showTargetTarget = {
						order = 4, 
						type = "toggle", 
						width = "full",
						name = L["Show Maintank Target-Target"],
						desc = L["Show Maintank Target-Target"],
						get = function() return GetOption("ShowTargetTarget") end,
						set = function(_,v) tempConfig["ShowTargetTarget"] = v 
							--WTFrames:UpdateConfig()
						end,
					},
					showRaidIcons = {
						order = 5, 
						type = "toggle", 
						width = "full",
						name = L["Show Raid Icons"],
						desc = L["Show Raid Icons"],
						get = function() return GetOption("ShowRaidIcons") end,
						set = function(_,v) tempConfig["ShowRaidIcons"] = v 
							--WTFrames:UpdateConfig()
						end,
					},			
					barYOffset = {
						order = 6, 
						type = "range", 
						min = 0,
						max = 50,
						step = 0.1,
						width = "full",
						name = L["Bar Y-Offset"],
						desc = L["Bar Y-Offset"],
						get = function() return GetOption("YOffset") end,
						set = function(_,v) tempConfig["YOffset"] = v 
							WTFrames:UpdateConfigOTF()
						end,
					},		
					barHeight = {
						order = 7, 
						type = "range", 
						min = 10,
						max = 100,
						step = 1,
						width = "full",
						name = L["Bar height"],
						desc = L["Bar height"],
						get = function() return GetOption("BarHeight") end,
						set = function(_,v) tempConfig["BarHeight"] = v 
							--WTFrames:UpdateConfig()
						end,
					},					
					barWidth = {
						order = 8, 
						type = "range", 
						min = 50,
						max = 300,
						step = 1,
						width = "full",
						name = L["Bar width"],
						desc = L["Bar width"],
						get = function() return GetOption("BarWidth") end,
						set = function(_,v) tempConfig["BarWidth"] = v 
							--WTFrames:UpdateConfig()
						end,
					},				
					powerBarHeightFactor = {
						order = 9, 
						type = "range", 
						min = 0,
						max = 1,
						step = 0.01,
						width = "full",
						name = L["PowerBar height factor"],
						desc = L["PowerBar height factor"],
						disabled = function()
							return not GetOption("ShowPowerBars")
						end,
						get = function() return GetOption("PowerBarHeightFactor") end,
						set = function(_,v) tempConfig["PowerBarHeightFactor"] = v 
							--WTFrames:UpdateConfig()
						end,
					},		
					iconSize = {
						order = 10, 
						type = "range", 
						min = 5,
						max = 32,
						step = 1,
						width = "full",
						name = L["Icon size"],
						desc = L["Icon size"],
						disabled = function() 
							return not GetOption("ShowRaidIcons")
						end,
						get = function() return GetOption("IconSize") end,
						set = function(_,v) tempConfig["IconSize"] = v 
							--WTFrames:UpdateConfig()
						end,
					},				
					textureName = {
						order = 11,
						type = "select",
						name = L["Bar texture"],
						desc = L["Bar texture"],
						dialogControl = "LSM30_Statusbar",
						values = WTFrames:GetTextures(),
						get = function() return GetOption("TextureName") end,
						set = function(_,v) tempConfig["TextureName"] = v 
							WTFrames:UpdateConfigOTF()
						end,					
					},
					fontName = {
						order = 12,
						type = "select",
						name = L["Font"],
						desc = L["Font"],
						dialogControl = "LSM30_Font",
						values = WTFrames:GetFonts(),
						get = function() return GetOption("FontName") end,
						set = function(_,v) tempConfig["FontName"] = v 
							WTFrames:UpdateConfigOTF()
						end,					
					},
					fontSize = {
						order = 13, 
						type = "range", 
						min = 9,
						max = 20,
						step = 1,
						width = "full",
						name = L["Font size"],
						desc = L["Font size"],
						get = function() return GetOption("FontSize") end,
						set = function(_,v) tempConfig["FontSize"] = v 
							--WTFrames:UpdateConfig()
						end,
					},	
					applyButton = {
						order = 14,
						type = "execute",
						name = L["Apply changes (PREVIEW)"],
						desc = "MT Frames will be re-created to apply your settings! EXPERIMENTAL! WARNING! This Button will create additional Frames which cannot be removed. This will affect your FPS/Load. Please reload your UI before doing serious stuff (Raid, PVP, ...)",
						disabled = function() return InCombatLockdown() end,
						func = function() 
							for k,v in pairs(tempConfig) do 
								WTFramesConfig.options[k] = GetOption(k)
							end
							
							WTFrames:UpdateConfig()
						end,
					},				
					reloadButton = {
						order = 15,
						type = "execute",
						name = L["Reload UI"],
						desc = L["Reload UI"],
						disabled = function() return InCombatLockdown() end,
						func = function() 
							for k,v in pairs(tempConfig) do 
								WTFramesConfig.options[k] = GetOption(k)
							end
							
							WTFrames:UpdateConfig()
						
							ReloadUI()
						end,
					}					
				}
			},
		}
	}
	
	return options
end

oUF.Tags.Events["wtf:exthp"] = "UNIT_HEALTH"
oUF.Tags.Methods["wtf:exthp"] = function(unit)
	return UnitIsDead(unit) and "Dead" 
	    or UnitIsGhost(unit) and "Ghost" 
	    or not UnitIsConnected(unit) and "Offline" 
		or string.format('%d%%', floor(UnitHealth(unit) / UnitHealthMax(unit) * 100))
end

oUF.Tags.Events["wtf:truncname"] = "UNIT_TARGET"
oUF.Tags.Methods["wtf:truncname"] = function(unit)
	local name = UnitName(unit)
	if (string.len(name) > 12) then
		name = string.format("%s...", name:sub(1, 12))
	end
	
	return name
end

-- Target Border
local ChangedTarget = function(self)
	if (UnitInRaid'player' == 1 or GetNumPartyMembers() > 0 ) and UnitIsUnit('target', self.unit) then
		WTFrames:Print("Show Border")
		self.TargetBorder:Show()
	else
		WTFrames:Print("Hide Border")
		self.TargetBorder:Hide()
	end
end

function WTFrames:chatCommand(input) 	
	local cmd = WTFrames:GetArgs(input)
	
	if(cmd == "" or not cmd)then		
		LibStub("AceConfigDialog-3.0"):Open("WTFrames")
	else 
		LibStub("AceConfigCmd-3.0").HandleCommand(WTFrames, "wtframes", "WTFrames", input)
	end
end

function WTFrames:OnInitialize()
	SML = LibStub:GetLibrary("LibSharedMedia-3.0")

	if not WTFramesConfig or WTFramesConfig.configVersion == nil then
		InitConfig()
	else
		-- Upgrade Config 2 -> 3, Add Font options
		if(WTFramesConfig.configVersion == 2 and currentConfigVersion == 3) then
			WTFramesConfig.options.FontName = "WTFramesDefault"
			WTFramesConfig.options.FontSize = 11
			WTFramesConfig.configVersion = 3
		elseif(WTFramesConfig.configVersion == 3 and currentConfigVersion == 4) then
			WTFramesConfig.options.GrowLeft = false
			WTFramesConfig.configVersion = 4
		elseif WTFramesConfig.configVersion < currentConfigVersion then -- Unknown upgrade path		
			InitConfig()
		end
	end
		
	-- Default bar texture
	SML:Register(mTypeTexture, 'WTFramesDefault', 'Interface\\Addons\\WTFrames\\media\\statusbar')
	
	-- Default font
	SML:Register(mTypeFont, 'WTFramesDefault', 'Interface\\Addons\\WTFrames\\media\\default.ttf')
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("WTFrames", loadOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("WTFrames")
	initLDB()
	
	WTFrames:RegisterChatCommand("wtframes", "chatCommand")
	
end

function WTFrames:UpdateConfigOTF()
	local statusBarTexture = SML:Fetch(mTypeTexture, GetOption("TextureName"))
	-- local GrowUpwards = WTFramesConfig.options.GrowUpwards;	
	-- local yOffsetAbsolute = WTFramesConfig.options.YOffset;
	
	-- local yOffset
	-- local localAnchorPoint
	-- local sortDir

	for k, v in pairs(TankFrames) do
		v.Health:SetStatusBarTexture(statusBarTexture) 
		if not (v.Power == nil) then
			v.Power:SetStatusBarTexture(statusBarTexture) 
		end
	end
	
	-- if GrowUpwards then
		-- yOffset = (-1) * yOffsetAbsolute
		-- localAnchorPoint = "TOP"
		-- sortDir = "ASC"
		-- maintankFrame:SetPoint("BOTTOMLEFT", anchorRef, "TOPLEFT", 0, 0)
	-- else
		-- yOffset = yOffsetAbsolute
		-- localAnchorPoint = "BOTTOM"		
		-- sortDir = "DESC"
		-- maintankFrame:SetPoint("TOPLEFT", anchorRef, "BOTTOMLEFT", 0, 0)
	-- end
	
	-- maintankFrame:SetManyAttributes(
		-- "template", template,
		-- "yOffset", yOffset,
		-- "point", localAnchorPoint,
		-- "sortDir", sortDir
	-- )	
	
end
 
function WTFrames:UpdateConfig()
	if maintankFrame then
	
		--Ninja-Unhook ~everything, doesn't work :/
		
		for k, v in pairs(TankFrames) do
			v.Health:UnregisterAllEvents()
			v.Health:SetScript("OnShow", nil)
			v.Health:SetScript("OnEvent", nil)				
			
			if not (v.Power == nil) then
				v.Power:UnregisterAllEvents()
				v.Power:SetScript("OnShow", nil)
				v.Power:SetScript("OnEvent", nil)				
			end	
			
			v.textName = nil
			v.textHP = nil
			
			v:UnregisterAllEvents()
			v:SetScript("OnShow", nil)
			v:SetScript("OnEvent", nil)
			
			-- Hide TT stuff
			if(v.isChild) then
				v.Show = function() end
				v:Hide()
				v:SetAttribute("useparent-unit", false)
				v:SetWidth(0)
				v:SetHeight(0)				
			end			
			
			-- remove globals frame reference to prevent parenting
			_G[v:GetName()] = nil
		end
	
		maintankFrame:UnregisterAllEvents()
		maintankFrame:SetScript("OnAttributeChanged", nil)
		maintankFrame:SetScript("OnShow", nil)
		maintankFrame:SetScript("OnEvent", nil)		
		maintankFrame.Show = function() end
		maintankFrame:Hide()
	end	
	
	WTFrames:CreateMTTFrames()
	
	maintankFrame:Show()
end

function WTFrames:GetTextures()
	for k in pairs(textures) do textures[k] = nil end

	for _, name in pairs(SML:List(SML.MediaType.STATUSBAR)) do
		textures[name] = name
	end
	
	return textures
end

function WTFrames:GetFonts()
	for k in pairs(fonts) do fonts[k] = nil end

	for _, name in pairs(SML:List(SML.MediaType.FONT)) do
		fonts[name] = name
	end
	
	return fonts
end

 
function initLDB()
	if LDB then
		local launcher = LibStub("LibDataBroker-1.1"):NewDataObject("WTFrames", {
			type = "launcher",
			label = "WTFrames",
			icon = "Interface\\Icons\\Ability_Warrior_DefensiveStance",
			OnClick = function(self, button)
				if button == "LeftButton" then
					--if InCombatLockdown() then return end
					if  anchorRef:IsVisible() then
						WTFrames:Lock()
					else
						WTFrames:UnLock()
					end
				else	
					for i, button in ipairs(InterfaceOptionsFrameAddOns.buttons) do
						if button.element and button.element.name == "WTFrames" and button.element.collapsed then
							OptionsListButtonToggle_OnClick(button.toggle)
						end
					end									
					if LibStub("AceConfigDialog-3.0").OpenFrames["WTFrames"] then
						LibStub("AceConfigDialog-3.0"):Close("WTFrames")						
					else
						LibStub("AceConfigDialog-3.0"):Open("WTFrames")
					end	
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("Who's tanking - Frames")
				tooltip:AddLine("|cffffff00" .. L["Left-click to toggle anchor."] .. "|r")
				tooltip:AddLine("|cffffff00" .. L["Right-click for options."] .. "|r")
			end
		})		
	end
end

function WTFrames:IsLocked() 
	return not anchorRef:IsVisible()
end

function WTFrames:Lock()
	if not WTFrames:IsLocked() then
		WTFrames:Print(L["Maintank frames are now locked"])
		anchorRef:Hide()
	end
end

function WTFrames:UnLock()
	if WTFrames:IsLocked() then
		WTFrames:Print(L["Maintank frames are now unlocked"])
		anchorRef:Show()
	end
end

function WTFrames:Reset()	
	InitConfig()
end

function oRA3MaintankStyle(self, unit)
	local frameheight = WTFramesConfig.options.BarHeight
	local framewidth = WTFramesConfig.options.BarWidth
	local powerbarheightfactor = WTFramesConfig.options.PowerBarHeightFactor	
		
	local statusBarTexture = SML:Fetch(mTypeTexture, WTFramesConfig.options.TextureName)
	
	local showPowerBars = WTFramesConfig.options.ShowPowerBars
	local iconSize = WTFramesConfig.options.IconSize
	
	local font = SML:Fetch(mTypeFont,  WTFramesConfig.options.FontName)	
	local fontSize = WTFramesConfig.options.FontSize
	
	local powerbarheight
	
	if showPowerBars then
		powerbarheight = frameheight * powerbarheightfactor
	else
		powerbarheight = 0
	end
	
	local healthbarheight = frameheight - powerbarheight
	
	self:SetWidth(framewidth)
	self:SetHeight(frameheight)
		
	local hp = CreateFrame(mTypeTexture, "HealthBar", self) 
    hp:SetPoint("TOPLEFT") 
    hp:SetStatusBarTexture(statusBarTexture) 
	hp:SetWidth(framewidth)
    hp:SetHeight(healthbarheight) 
    hp:SetPoint("RIGHT") 
    hp:SetPoint("LEFT")
	hp:SetFrameLevel(unit and 1 or 2)
	
	
    hp.bg = hp:CreateTexture(nil, "BORDER")
    hp.bg:SetAllPoints(hp)
    hp.bg:SetTexture(statusBarTexture)
   	hp.bg.multiplier = 0.3
	
	hp.colorReaction = true
	hp.colorClass = true
	hp.colorClassNPC = false
	hp.colorDisconnected = true
	hp.colorTapping = true	
 	hp.frequentUpdates = true
	hp.colorSmooth = true	
		
	local textHP = hp:CreateFontString(nil, 'OVERLAY')
	textHP:SetFont(font, fontSize or 12)

	textHP:SetFontObject(GameFontNormalSmall)
	textHP:SetShadowColor(0,0,0)
	textHP:SetShadowOffset(0.8, -0.8)
	textHP:SetTextColor(1,1,1)
	textHP:SetJustifyH("LEFT")	
	textHP:SetPoint('RIGHT',self,'RIGHT',-2,2)
	textHP.frequentUpdates = true	   	
    self:Tag(textHP,'[wtf:exthp]')
				
	local textName = hp:CreateFontString(nil, 'OVERLAY')
	-- textName:SetFontObject(GameFontNormalSmall)
	textName:SetFont(font, fontSize or 12)
	
	textName:SetShadowColor(0,0,0)
	textName:SetShadowOffset(0.8, -0.8)
	textName:SetTextColor(1,1,1)
	textName:SetJustifyH("LEFT")	
	textName:SetPoint('LEFT',self,'LEFT',17,2)
	textName.frequentUpdates = true	   	
    self:Tag(textName,'[wtf:truncname]')
	
	self.Health = hp   	
	
	if showPowerBars then
		local pp = CreateFrame('StatusBar', nil, self)
		pp:SetHeight(powerbarheight - 1)
		pp:SetStatusBarTexture(statusBarTexture)
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")
		pp:SetPoint("TOP", hp, "BOTTOM", 0, 0)

		pp.colorDisconnected = true
		pp.colorPower = true
		pp.frequentUpdates = true
		pp.colorSmooth = true
		
		pp.bg = pp:CreateTexture(nil, "BORDER")
		pp.bg:SetAllPoints(pp)
		pp.bg:SetTexture(statusBarTexture)
		pp.bg:SetAlpha(.5)
		pp.bg.multiplier = 0.3
		
		self.Power = pp	
	end
		  
    if(WTFramesConfig.options.ShowRaidIcons)then
        local raidIcon = hp:CreateTexture(nil, "OVERLAY")
        raidIcon:SetPoint("LEFT", self, 0, 2)
        raidIcon:SetHeight(iconSize)
        raidIcon:SetWidth(iconSize)
        raidIcon.frequentUpdates = 0.1		
		
		self.RaidIcon = raidIcon
    end
	
	-- self:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			-- tile = true,
			-- tileSize = 16,
			-- edgeFile = SML:Fetch(mTypeBorder, borderTexture),
			-- edgeSize = 1,
			-- insets = { left = 5, right = 5, top = 5, bottom = 5 }
	-- })
	-- self:SetBackdropColor(0, 0, 0, 0)
	local tBorder = self:CreateTexture(nil, "OVERLAY")
	tBorder:SetPoint("LEFT", self, "LEFT", -6, 0)
	tBorder:SetPoint("RIGHT", self, "RIGHT", 6, 0)
	tBorder:SetPoint("TOP", self, "TOP", 0, 6)
	tBorder:SetPoint("BOTTOM", self, "BOTTOM", 0, -6)
	tBorder:SetTexture(borderTexture)
	tBorder:Hide()
	tBorder:SetVertexColor(.8, .8, .8, .8)
	--self.TargetBorder = tBorder

	--self:RegisterEvent('PLAYER_TARGET_CHANGED', ChangedTarget)
	
	WTFrames:RegisterTankFrame(self)
	
	return self
 end
 
function WTFrames:RegisterTankFrame(frame)
	table.insert(TankFrames,frame)
end


function WTFrames:GetMaintankList(onlyShowBlizzardTanks)
	local oRA3Tanks = oRA3:GetSortedTanks()	
	
	local res = {}
	
	if onlyShowBlizzardTanks then
		local blizzTanks = oRA3:GetBlizzardTanks()
	
		for k in pairs(oRA3Tanks) do 
			local tank = oRA3Tanks[k]
		
			for kb in pairs(blizzTanks) do
				if tank == blizzTanks[kb] then
					table.insert(res, tank)
				end
			end
		end
	else
		res = oRA3Tanks
	end
	
	return res
end

function WTFrames:UpdateMaintankList()	
	local tanks = WTFrames:GetMaintankList(WTFramesConfig.options.OnlyShowBlizzardTanks)

	maintankFrame:SetAttribute("nameList", table.concat(tanks, ","))
end
 
function WTFrames:CreateMTTFrames()
	-- Some local attributes
	local GrowUpwards = WTFramesConfig.options.GrowUpwards;	
	local GrowLeft = WTFramesConfig.options.GrowLeft;	
	
	local yOffsetAbsolute = WTFramesConfig.options.YOffset;
	local showTT = WTFramesConfig.options.ShowTargetTarget;
	
	local yOffset
	local localAnchorPoint
	local sortDir
	local template
	
	if showTT then
		template = "WTFrames_Maintank"
	else
		template = "WTFrames_Maintank_TargetOnly"
	end
	
	if GrowLeft then
		template = template.."_Left"
	end
	
	local oRA3Tanks = oRA3:GetSortedTanks()
	
	--local maintanks = oUF:SpawnHeader("WTFrames_MainTanks", nil, 'raid')
	
	local anchorR 
	local anchorL
	
	if GrowUpwards then
		yOffset = (-1) * yOffsetAbsolute
		localAnchorPoint = "TOP"
		sortDir = "DESC"
		--maintanks:SetPoint("BOTTOMLEFT", anchorRef, "TOPLEFT", 0, 0)
		
		anchorR = "BOTTOMLEFT"
		anchorL = "TOPLEFT"
	else
		yOffset = yOffsetAbsolute
		localAnchorPoint = "BOTTOM"		
		sortDir = "DESC"
		--maintanks:SetPoint("TOPLEFT", anchorRef, "BOTTOMLEFT", 0, 0)
		anchorR = "TOPLEFT"
		anchorL = "BOTTOMLEFT"		
	end
		
	--maintanks:SetManyAttributes(		
		--"nameList", table.concat(oRA3Tanks, ","),
		--"template", template,
		--"showRaid", true,
		--"initial-unitWatch", true,
		--"yOffset", yOffset,
		--"point", localAnchorPoint,
--		"sortDir", sortDir
		--"filter", "MAINTANK"
	--)
	
	local maintanks = oUF:SpawnHeader("WTFrames_MainTanks", nil, 'raid', 
		"nameList", table.concat(oRA3Tanks, ","),
		"template", template,
		"showRaid", true,
		"initial-unitWatch", true,
		"yOffset", yOffset,
		"point", localAnchorPoint,
		"sortDir", sortDir
	)
	
	--maintanks:SetPoint("TOPLEFT", anchorRef, "BOTTOMLEFT", 0, 0)
	maintanks:SetPoint(anchorR, anchorRef, anchorL, 0, 0)
	
	maintankFrame = maintanks
	
	
end
 
function WTFrames:OnEnable()	
	oUF:RegisterStyle('oRA3MaintankStyle', oRA3MaintankStyle) 
	oUF:SetActiveStyle("oRA3MaintankStyle")	
	
	local anchor = CreateFrame("Frame", nil, UIParent)
	local anchorText = anchor:CreateFontString(nil, nil, "GameFontNormalSmall")
	anchorText:SetPoint("TOPLEFT")
	anchorText:SetText("WTFrames Anchor")
	anchor:SetWidth(WTFramesConfig.options.BarWidth)
	--anchor:SetHeight(24 + (#oRA3Tanks * (WTFramesConfig.options.FrameHeight + 2.9)))
	anchor:SetHeight(12)

	--anchor:SetBackdrop(backdrop)
	--anchor:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
	--anchor:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)		
	
	anchor:SetPoint(WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y )
	
	anchor:EnableMouse(true)
	anchor:SetMovable(true)
	anchor:SetScript("OnMouseDown", anchor.StartMoving)
	anchor:SetScript("OnMouseUp", function(self, button)
		self:StopMovingOrSizing()
		WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y = "BOTTOMLEFT", self:GetLeft(), self:GetBottom()
	end)
	
	anchor:Hide()
			
	anchorRef = anchor
	
	WTFrames:CreateMTTFrames()
	
	local tankhandler = {}
	function tankhandler:OnTanksUpdated(event, tanks) 
		WTFrames:UpdateMaintankList()		
	end
	function tankhandler:OnTanksChanged(event, tanks) 
		WTFrames:UpdateMaintankList()		
	end			
	
	oRA3.RegisterCallback(tankhandler, "OnTanksUpdated")
	oRA3.RegisterCallback(tankhandler, "OnTanksChanged")	
	
	WTFrames:UpdateMaintankList()
		
	maintankFrame:Show()
end

